/***************************************************************************
                          bicompdata.h  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BICOMPDATA_H
#define BICOMPDATA_H

#include "node.h"
#include <list>

namespace graphlib {

/**
  * @short node-datas needed for the biconnectivity test.
  * @see BiComponents
  */

class BiCompData
{
  	//------------------------------------------------------ friend functions
    /**
      * Output-operator.
      * Writes the datas onto the output-stream.
      *
      * @param os the output-stream
      * @param bc the datas
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os, const BiCompData& bc);

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Default Constructor. Creates default data for Biconntectivity
      * Algorithm: dnf = low = 0, unmarked, no father;
      *
      * @see BiComponents
      */
    BiCompData();

  	//------------------------------------------------------ Information
  	/**
  	  * Returns true, wenn marked == true.
  	  */
  	bool isMarked() const { return marked; }
  	
  	/**
  	  * @return low.
  	  */
  	unsigned getLow() const { return low; }
  	
  	/**
  	  * @return dnf.
  	  */
  	unsigned getDNF() const { return dnf; }
  	
  	/**
  	  * @return father.
  	  */
  	Node getFather() const {return father; }
  	
  	//------------------------------------------------------ set function
  	/**
  	  * Sets <code>marked</code>, <code>dnf</code> and <code>low</code>.
  	  *
  	  * @param marked
  	  * @param dnf
  	  * @param low
  	  * @return itself.
  	  */
  	BiCompData& setBiCompData(bool marked, unsigned dnf, unsigned low);
  	/**
  	  * Sets <code>marked</code>, <code>dnf</code>, <code>low</code> and
  	  * <code>father</code>.
  	  *
  	  * @param marked
  	  * @param dnf
  	  * @param low
  	  * @param father
  	  * @return itself.
  	  */
  	BiCompData& setBiCompData(bool marked, unsigned dnf, unsigned low,
  	                   const Node& father);

  	/**
  	  * Changes <code>low</code>.
  	  *
      * @param low
  	  * @return itself.
  	  */
    BiCompData& changeLow(unsigned low);
  	

  private:	
  	//------------------------------------------------------ datas
    /// if node marked -> true
    bool               marked;
    /// depth-first number
    unsigned           dnf;
    /// low-number
    unsigned           low;
    /// father of the node
    Node               father;
};  // class BiCompData

} // namespace graphlib
#endif  // BICOMPDATA_H

/* end of file */
