/***************************************************************************
                          cyclegraph.cpp  -  description
                             -------------------
    begin                : Mon May 21 2001
    cou=unodes.begin();pyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "cyclegraph.h"

#include "trace.h"
#include "logfile.h"

#include "utilities.h"


#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
CycleGraph::CycleGraph() : Graph()
{;}



//---------------------------------------------------------------------------
// private functions for calculations
//---------------------------------------------------------------------------
NodePathMap2& CycleGraph::dijkstra(Node node, NodePathMap2& nodeMap,
                                  NodeSet& nodeSet, bool ordered) const
{
  #if GRAPHLIB_DEBUG==6
    static bool ersteMal=true;
    LogFile dijk;
    if (ersteMal)
    {
      dijk.init("dijkstra.graphlib");
      ersteMal=false;
    }
    else
      dijk.reopen("dijkstra.graphlib");
    #define DIJK(msg) dijk.logging(msg);
  #else
    #define DIJK(msg) ;
  #endif

  constEdgeIter edgeI;
  Node neighb;
  NodePathMap2::iterator nodeI = nodeMap.find(node);
  NodePathMap2::iterator neighbI;
  unsigned dist = 0;
  int callingNode = node.id();

  DIJK("calling Node:"+Ttos(callingNode)+" order: "+Ttos(ordered))

  for (;;)
  {
    // distance to new nodes
    dist = nodeI->second.distance + 1;

    #if GRAPHLIB_DEBUG==6
      writeNodeMap(nodeMap,dijk.os());
    #endif
    DIJK("\n\n node: "+Ttos(node)+"\nneighbours: ")

    // for all neighbours
    for (edgeI=node.adjBegin(); edgeI!=node.adjEnd(); edgeI++)
    {
      neighb = node.opposite(*edgeI);
      neighbI = nodeMap.find(neighb);
      DIJK(Ttos(neighb)+" ")

      // neighbour is in this biconnected component
      // and not marked (no shortest path was found till now)
      if (neighbI!=nodeMap.end() && ! neighbI->second.marked)
      {
        // there is already a path to the neighbour, cause distance != inf
        if (neighbI->second.distance < UINT_MAX)
        {
          // a shorter distance is found
          if (nodeI->second.distance > dist)
          {
            // set new distance
            neighbI->second.distance = dist;

            // if order important and
            // neighbour greater or current's path emtpy -> delete path
            if (ordered &&
                (neighb.id()>callingNode || nodeI->second.path.empty()))
              neighbI->second.path.clear();

            // else neighbour's path = currents path + current node
            else
            {
              neighbI->second.path = nodeI->second.path;
              neighbI->second.path.push_back(neighb);
            }
          }

          // same distance
          else if (neighbI->second.distance == dist)
          {
            // if order important and current's path not empty
            if (ordered && !nodeI->second.path.empty())
            {
              // neighbour greater -> delete path
              if (neighb.id()>callingNode)
                neighbI->second.path.clear();

              // else neighbour's path = currents path + current node
              else
              {
                neighbI->second.path = nodeI->second.path;
                neighbI->second.path.push_back(neighb);
              }
            }
          }
        }

        // new node found - there was no path to this node till now
        else
        {
          // set new distance
          neighbI->second.distance = dist;

          // if order important and
          // neighbour greater or current's path emtpy -> delete path
          if (ordered &&
              (neighb.id()>callingNode || nodeI->second.path.empty()))
            neighbI->second.path.clear();

          // else neighbour's path = currents path + current node
          else
          {
            neighbI->second.path = nodeI->second.path;
            neighbI->second.path.push_back(neighb);
          }
        }
      }
    }

    // mark current node
    nodeI->second.marked = true;
    nodeSet.erase(node);

    // new node = node with smallest distance
    if (!nodeSet.empty())
    {
      node = findNodeWithSmallestDistance(nodeMap,nodeSet);
      nodeI = nodeMap.find(node);
    }
    // END
    else
    {
      DIJK("\n Dijkstra Ende *************************\n")
      return nodeMap;
    }
  }
}

void CycleGraph::initNodeMap(const Node& node, const Graph& biComp,
                             NodePathMap2& nodeMap, NodeSet& nodeSet) const
{
  constNodeIter it=biComp.nodesBegin();
  constNodeIter end=biComp.nodesEnd();

  // distance = INF, marked = false, path = {node}
  NodePath data(node);

  // clears map and set
  nodeMap.clear();
  nodeSet.clear();

  // all nodes of the graph
  while (it != end)
  {
    nodeMap.insert(nodeMap.end(),make_pair(*it,data));
    nodeSet.insert(nodeSet.end(),*it);
    it++;
  }

  // set for calling node distance = 0
  nodeMap[node].distance = 0;
}

Node CycleGraph::findNodeWithSmallestDistance(const NodePathMap2& nodeMap,
                              const NodeSet& nodeSet) const
{
  NodeSet::const_iterator it=nodeSet.begin();
  NodeSet::const_iterator end=nodeSet.end();

  Node result=*it;

  // all nodes in the set
  while (it != end)
  {
    if (nodeMap.find(result)->second.distance >
        nodeMap.find(*it)->second.distance)
      result = *it;

    it++;
  }
  return result;
}



void CycleGraph::writeNodeMap(const NodePathMap2& nodeMap, ostream& os) const
{
  NodePathMap2::const_iterator it  = nodeMap.begin();
  NodePathMap2::const_iterator end = nodeMap.end();

  os << endl;
  while (it != end)
  {
    os << it->first << "\t";
    os << it->second.distance << " ";
    os << it->second.marked;
    os << "\tpath :" << it->second.path << endl;
    it++;
  }
  os << "-------------------------\n";
}

} // namespace graphlib

/* end of file */
