/* $Id: edge.cpp,v 1.1.1.1 2005/01/04 15:52:03 xtof Exp $ */
/***************************************************************************
                          edge.cpp  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "edge.h"
#include "nodedata.h"
#include "edgedata.h"
#include "trace.h"

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Edge::Edge() : data(0)
{
}
//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
bool Edge::operator==(const Edge& e) const
{
  return (data == e.data);
}
bool Edge::operator!=(const Edge& e) const
{
  return (data != e.data);
}
bool Edge::operator<(const Edge& e) const
{
  return (data->id < e.data->id);
}

//---------------------------------------------------------------------------
// Input / Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os, const Edge& e)
{
  return os << e.source().id() << "-->" << e.target().id();
}

//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
unsigned Edge::id() const
{
  return data->id;
}

string Edge::label() const
{
  return data->label;
}

constEdgeIter Edge::pos() const
{
  return data->pos;
}

Node Edge::source() const
{
  return *(data->nodes.begin());
}

Node Edge::target() const
{
  return *(++(data->nodes.begin()));
}

void Edge::removeFrom() const
{
  NodeIter theNodes    = data->nodes.begin();
  NodeIter theNodesEnd = data->nodes.end();

  list<EdgeIter>::iterator theAdjPos;

  theAdjPos = data->adj_pos.begin();

  while(theNodes != theNodesEnd)
  {
    theNodes->data->edges.erase(*theAdjPos);
    theNodes->data->degree--;

    ++theNodes;
    ++theAdjPos;
  }
}

//---------------------------------------------------------------------------
// Set Function(s)
//---------------------------------------------------------------------------
constEdgeIter Edge::label(string label) {
  data->label = label;
  return data->pos;
}

} // namespace graphlib

/* end of file */
