/* $Id: edge.h,v 1.1.1.1 2005/01/04 15:52:03 xtof Exp $ */
/***************************************************************************
                          edge.h  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EDGE_H
#define EDGE_H

#include <list>
#include <functional>
#include <string>
#include <ostream>

using namespace std;

namespace graphlib {

class Node;
class EdgeData;

class Edge;
typedef list<Edge>::const_iterator  constEdgeIter;
typedef list<Edge>::iterator        EdgeIter;

/**
  *@short An edge in a graph
  */

class Edge
{
  	//------------------------------------------------------ friends
  friend ostream& operator<< (ostream&, const Edge&);
  	
  	//------------------------------------------------------ friends
  friend class Graph;
  friend class GML_Graph;
  friend class Node;

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Default constructor. Creates an invalid edge.
      * The only way to obtain a valid edge is through
      * @ref Graph#newEdge
      * Example:
      * <pre>
      * Graph g;
      * Node n1, n2;
      * Edge e;
      *
      * n1 = g.newNode();
      * n2 = g.newNode();
      * e  = g.newEdge(n1,n2);
      * </pre>
      *
      * @see Graph#newEdge
      */
  	Edge();

  	//------------------------------------------------------ Operators
  	/**
  	  * Return true, when <code>e</code> has same data adress.
  	  */
  	bool operator==(const Edge &e) const;
  	/**
  	  * Return true, when <code>e</code> has not same data adress.
  	  */
  	bool operator!=(const Edge &e) const;
  	/**
  	  * Return true, when id of <code>e</code>-data is lower.
  	  */
  	bool operator<(const Edge &e) const;
  	 	
  	//------------------------------------------------------ Information
  	/**
  	  * Returns id of the edge.
  	  */
  	unsigned id() const;
  	
  	/**
  	  * Returns label of the edge.
  	  */
  	string label() const;
  	
	/**
	  * Set label of the edge.
	  */
	constEdgeIter label(string label);

  	/**
  	  * Returns the position of list of all edges.
  	  */
  	constEdgeIter pos() const;
  	
  	/**
  	  * Returns the source node of the edge.
  	  */
  	Node source() const;
  	
  	/**
  	  * Returns the target node of the edge.
  	  */
  	Node target() const;
  	  	  	  	
  private:
    EdgeData *data;

    /**
      * Remove edge form adjacency list of source and target
      */
    void removeFrom() const;

}; // class Edge

} // namespace graphlib
#endif  // EDGE_H

/* end of file */
