/***************************************************************************
                          Gml2Smi.h  -  description
                             -------------------
    begin                : Mon Nov 05 2001
    copyright            : (C) 2001 by Gil Benkoe
    email                : gil@tbi.univie.ac.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GML2SMI_H
#define GML2SMI_H

#include "graph.h"
#include "utilities.h"
//#include "DFSIterator.h"

//#include <fstream>
#include <string>
#include <map>
#include <list>
#include <vector>

namespace graphlib {

typedef map<Node, string >        NodeStringMap;
typedef map<Node, int >        NodeIntMap;
typedef map< Node , list<Node> > NodePathMap;

#define WHITE 0
#define GRAY 1
#define BLACK 2

 int dfs_main(const Node&, list<Node>&, NodeStringMap&, NodePathMap&);  

 namespace {
   NodeIntMap color;
   int ringNb;
   constNodeIter last;
   int geschwungenNb;
   bool withH;
 }

string gml2smi (const Graph& g, bool withH_ = 1) {



  if (!g.numberNodes()) { cerr << "graphlib::gml2smi(Graph&) : No nodes in graph" << endl; return ""; }

  constNodeIter vi = g.nodesBegin();
  // if H are to be implicit, jump to first non-H node
  while (!withH_ && vi->label() == "H")
    vi++;

  // initialize for DFS
  list<Node> tmp;
  vector< list<Node> > ordered(4, tmp); 
  string smiles = "";
  NodeStringMap nodeMap;
  NodePathMap geschwungen;

  nodeMap[*vi] = vi->label();

  ringNb = 1;
  last = vi->pos();
  geschwungenNb = 0;
  color.clear();
  withH = withH_;
  
  unsigned int i;	
  for (i = 0 ; i < 100 ; i++)
  {
    if (dfs_main(*vi, ordered[i], nodeMap, geschwungen) != 0)
      THROW(runtime_error,"gml2smi(const Graph&) : error doing dfs")

    if (color.size() == g.numberNodes())
      break;

    // if H are to be implicit, jump to first non-H node
    while (color.count(*vi) || (!withH_ && vi->label() == "H"))
      vi++;
    
    ringNb = 1;
    last = vi->pos();
    geschwungenNb = 0;
    withH = withH_;
  }

  {
    NodePathMap::iterator it = geschwungen.begin();
    NodePathMap::iterator end = geschwungen.end();
    Node cur;

    while (it != end) {
      cur = it->first;
      NodeIter Nit = it->second.begin();
      NodeIter Nend = it++->second.end();  
      while (Nit != Nend) 
	if (nodeMap[*Nit++][0] == '(') 
	  nodeMap[cur] = nodeMap[cur] + ')';
    }
  }

  i++; 
  for (vector< list <Node> >::iterator ord = ordered.begin(); 
       i; ord++, i--)
  {
    if (ord != ordered.begin())
      smiles += ".";
    NodeIter it = ord->begin();
    NodeIter end = ord->end();
    while (it != end) 
      smiles += nodeMap[*it++];
  }

  return smiles;
}

int dfs_main(const Node& vertex,           //current vertex
	     list<Node>& ordered,          //vertices visited so far
	     NodeStringMap& nodeMap,       //smiles-tokens of vertices
	     NodePathMap& geschwungen) {   //used for determination of )'s



 int pos;
 

constEdgeIter      member, lastWhite = vertex.adjEnd();

/*****************************************************************************
*                                                                            *
*  Color the vertex gray and traverse its adjacency list.                    *
*                                                                            *
*****************************************************************************/

 color[vertex] = GRAY;
 ordered.push_back(vertex);

 //cout << vertex.label();

for (member = vertex.adjBegin(); member != vertex.adjEnd(); member++) {

   /**************************************************************************
   *                                                                         *
   *  Determine the color of the next adjacent vertex.                       *
   *                                                                         *
   **************************************************************************/

   const Node& adj_vertex = vertex.opposite(*member);

   //if H are to be implicit, ignore H nodes
   if (!withH && adj_vertex.label() == "H")
     continue;

   if (adj_vertex.adjBegin() == adj_vertex.adjEnd())
      return -1;


   /**************************************************************************
   *                                                                         *
   *  Move one vertex deeper when the next adjacent vertex is white.         *
   *                                                                         *
   **************************************************************************/

   if (!color.count(adj_vertex)) {
     //initialize last
     last = ordered.begin(); 
     //lastWhite is current white so far
     lastWhite = member; 

     //encountering adj_vertex for the first time 
     //-> initialize its name to edge(adj_vertex, vertex) + label
     nodeMap[adj_vertex] = (member->label() == "-" ? "" : member->label()) + adj_vertex.label() ;//fprintf(stdout, "("); 
     //add '(' if adj_vertex is a fork
     if (vertex.degree() > 2 || 
	(vertex == *ordered.begin() && vertex.degree() > 1) )  
       nodeMap[adj_vertex] = '(' + nodeMap[adj_vertex];
     //move
     if (dfs_main(adj_vertex, ordered, nodeMap, geschwungen) != 0)
         return -1;

   } else if (color[adj_vertex] == BLACK) {
     //if adj_vertex is black it won't be visited again
     //it is safe (it will be the only time) to indicate ring closure
     //by inserting an index in adj_vertex' name 
     //(right after the element symbol, before ')' )
     pos = nodeMap[adj_vertex].find_first_of(')');
     nodeMap[adj_vertex].insert( pos == -1 ? nodeMap[adj_vertex].size() : pos , 
				 ringNb > 9 ? "%" + Ttos(ringNb) : Ttos(ringNb) );
     nodeMap[vertex] = nodeMap[vertex] + (member->label() == "-" ? "" : member->label()) + ( ringNb > 9 ? "%" + Ttos(ringNb++) : Ttos(ringNb++) );
   }

}

//for better readability of smiles reduce C(C)(C) to C(C)C
//by erasing parentheses of lastWhite
 if (lastWhite != vertex.adjEnd() &&
     nodeMap[vertex.opposite(*lastWhite)][0] == '(') 
   nodeMap[vertex.opposite(*lastWhite)].erase(0,1);

/*****************************************************************************
*                                                                            *
*  Color the current vertex black and make it first in the list.             *
*                                                                            *
*****************************************************************************/

color[vertex] = BLACK;

//if this is the first time we're moving up (current vertex black, last is initialized)
//the current vertex is the dead end of the graph and will get all the )'s
//set last to the current vertex
 if (*last == *ordered.begin()) last = vertex.pos();     
 //if (nodeMap[vertex][0] == '(') nodeMap[*last] = nodeMap[*last] + ')';
 //for each vertex we're moving up, add it to geschwungen[last]
 geschwungen[*last].push_back(vertex);
 //fprintf(stdout, ")");
 //ordered.push_front(vertex);


return 0;

}

} // namespace graphlib


#endif // DOT2GML_H

/* end of file */
