/***************************************************************************
                          gml_graph.h  -  description
                             -------------------
    begin                : Wed Mar 7 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GML_GRAPH_H
#define GML_GRAPH_H

#include "graph.h"
#include "coordinates.h"

#include <string>
#include <fstream>
#include <sstream>

namespace graphlib {

/**
  *@short An undirected GML graph
  *
  * A graph created from the informations of a GML-file.
  *
  * @see Graph
  * @see
  *   <a href="http://www.infosun.fmi.uni-passau.de/Graphlet/GML/index.html">
  *   The GML File Format</a>
  */

class GML_Graph : public Graph {
  	

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Constructor. Creates a new parser. Bind the ifstream to
      * the GML-file <code>fname</code> and creates a new Graph.
      *
      * @param fname name of the GML-file
      */
  	GML_Graph(string fname);
   	 	
  	//------------------------------------------------------ Information
  	/**
  	  * Returns the graph generated from the GML-file.
  	  */
  	Graph* graph();
  	  	  	  	
  	/**
  	  * Returns the maximal x,y and z Coordinates of the graph.
  	  */
  	Coordinates maxCoord() { return maxCoords; }
  	
  	/**
  	  * Returns the minimal x,y and z Coordinates of the graph.
  	  */
  	Coordinates minCoord() { return minCoords; }
  	
  	
  private:
    //------------------------------------------------------ input stream
    /// input-stream
    ifstream in;
    /// filename
    string filename;

    //------------------------------------------------------ datas
    // maximal number of the x,y and z-coordinates
    Coordinates maxCoords;
    // minimal number of the x,y and z-coordinates
    Coordinates minCoords;
  	
  	//------------------------------------------------------ Utilities
    /// needed for generating the graph from the file
    void GML2Graph();
    /// needed for generating the graph from the file
    void findGraph();
    /// needed for generating the graph from the file
    void readNode();
    /// needed for generating the graph from the file
    void readCoordinates(Node*);
    /// needed for generating the graph from the file
    void readEdge();

}; // class GML_Graph

} // namespace graphlib
#endif // GML_GRAPH

/* end of file */
