/***************************************************************************
                          gml_parser.h  -  description
                             -------------------
    begin                :
    copyright            : (C) 1997 by Michael Himsolt
    email                : himsolt@fmi.uni-passau.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GML_PARSER_H
#define GML_PARSER_H

#include "gml_scanner.h"
#include <iostream>

#include <cstdlib>
#include <cassert>
#include <cstring>
//#include "utilities.h"

namespace graphlib {

/**
  @internal
  */
union GML_pair_val
{
    long integer;
    double floating;
    char* str;
    struct GML_pair* list;
};

/**
  @internal
  */
struct GML_pair
{
    char* key;
    GML_value kind;
    int lineNr;
    union GML_pair_val value;
    struct GML_pair* next;
};

/**
  @internal
  */
struct GML_list_elem
{
    char* key;
    int lineNr;
    struct GML_list_elem* next;
};

/**
  @internal
  */
struct GML_stat
{
    struct GML_error err;
    struct GML_list_elem* key_list;
};

/**
  * returns list of KEY - VALUE pairs. Errors and a pointer to a list
  * of key-names are returned in GML_stat. Previous information contained
  * in GML_stat, i.e. the key_list, will be *lost*.
  */
template <class X> struct GML_pair* GML_parser (X source, struct GML_stat* stat, int open); //moved from .h to .cpp by gil 09/07/02
     /*
{
  struct GML_token token;
  struct GML_pair* pair;
  struct GML_pair* list;
  struct GML_pair* tmp = NULL;
  struct GML_list_elem* tmp_elem;

  assert (stat);

  pair = (struct GML_pair*) malloc (sizeof (struct GML_pair));
  list = pair;

  for (;;)
  {
	  token = GML_scanner (source); // for GML_pair kind
	
	  // GML_END
	  if (token.kind == GML_END)
	  {
	    if (open)
	    {
	      stat->err.err_num = GML_OPEN_BRACKET;
    		stat->err.line = GML_line;
		    stat->err.column = GML_column;
		    free (pair);

    		if (tmp == NULL)
    		  return NULL;
    		else
    		{
		      tmp->next = NULL;
		      return list;
		    }
	    }
      break;
    }

    // GML_R_BRACKET
    else if (token.kind == GML_R_BRACKET)
    {
	    if (!open)
	    {
	      stat->err.err_num = GML_TOO_MANY_BRACKETS;
		    stat->err.line = GML_line;
    		stat->err.column = GML_column;
    		free (pair);

    		if (tmp == NULL)
    		  return NULL;
		    else
		    {
    		  tmp->next = NULL;
		      return list;
		    }
	    }
	    break;
    }

    // GML_ERROR
    else if (token.kind == GML_ERROR)
    {
      stat->err.err_num = token.value.err.err_num;
	    stat->err.line = token.value.err.line;
	    stat->err.column = token.value.err.column;
	    free (pair);
	      
	    if (tmp == NULL)
		    return NULL;
	    else
	    {
		    tmp->next = NULL;
		    return list;
	    }
    }

    // NOT GML_KEY -> wrong syntax in the file
    else if (token.kind != GML_KEY)
    {
	    stat->err.err_num = GML_SYNTAX;
	    stat->err.line = GML_line;
	    stat->err.column = GML_column;
	    free (pair);
	   
	    if (token.kind == GML_STRING)
		    free (token.value.str);

	    if (tmp == NULL)
		    return NULL;
	    else
	    {
		    tmp->next = NULL;
		    return list;
	    }
	  }
	   
	  // GML_KEY, GML_INT, GML_DOUBLE, GML_STRING, GML_L_BRACKET
	  // first time -> memory allocation, new key_list
	  if (!stat->key_list)
	  {
	    stat->key_list = (struct GML_list_elem*)
	          malloc (sizeof (struct GML_list_elem));
	    stat->key_list->next = NULL;
	    stat->key_list->key = token.value.str;
	    pair->key = token.value.str;
	    pair->lineNr = GML_line;
	  }
	
	  // key_list contains already some elements
	  else
	  {
	    tmp_elem = stat->key_list;
	    tmp_elem->lineNr = GML_line;
	    
	    while (tmp_elem)
	    {
		    if (!strcmp (tmp_elem->key, token.value.str))
		    {
		      free (token.value.str);
		      pair->key = tmp_elem->key;
		      pair->lineNr = tmp_elem->lineNr;
		      break;
		    }
		    tmp_elem = tmp_elem->next;
	    }
	
	    if (!tmp_elem)
	    {
		    tmp_elem = (struct GML_list_elem*)
		    malloc (sizeof (struct GML_list_elem));
    		tmp_elem->next = stat->key_list;
    		stat->key_list = tmp_elem;
    		tmp_elem->key = token.value.str;
    		pair->key = token.value.str;
  	    pair->lineNr = GML_line;
	    }
  	}
	
    token = GML_scanner (source); // for GML_pair value
    pair->lineNr = GML_line;
	
  	switch (token.kind)
  	{
   	case GML_INT:
 	    pair->kind = GML_INT;
 	    pair->value.integer = token.value.integer;
 	    break;

   	case GML_DOUBLE:
 	    pair->kind = GML_DOUBLE;
 	    pair->value.floating = token.value.floating;
 	    break;

   	case GML_STRING:
 	    pair->kind = GML_STRING;
 	    pair->value.str = token.value.str;
 	    break;

   	case GML_L_BRACKET:
 	    pair->kind = GML_LIST;
 	    pair->value.list = GML_parser (source, stat, 1);
     	
 	    if (stat->err.err_num != GML_OK)
 	      return list;

 	    break;

   	case GML_ERROR:
 	    stat->err.err_num = token.value.err.err_num;
 	    stat->err.line = token.value.err.line;
 	    stat->err.column = token.value.err.column;
 	    free (pair);
     		
 	    if (tmp == NULL)
 	      return NULL;
 	    else
 	    {
 		    tmp->next = NULL;
 		    return list;
 	    }
 	    break;

   	default:
 	    stat->err.line = GML_line;
 	    stat->err.column = GML_column;
 	    stat->err.err_num = GML_SYNTAX;
 	    free (pair);

 	    if (tmp == NULL)
 	      return NULL;
 	    else {
     		tmp->next = NULL;
     		return list;
 	    }
    }

  	tmp = pair;
  	pair = (struct GML_pair*) malloc (sizeof (struct GML_pair));
  	tmp->next = pair;
  }

  stat->err.err_num = GML_OK;
  free (pair);

  if (tmp == NULL)
  	return NULL;
  else
  {
  	tmp->next = NULL;
  	return list;
  }
}
*/

/**
  * free memory used in a list of GML_pair
  */
void GML_free_list (struct GML_pair*, struct GML_list_elem*);


/**
  * debugging
  */
void GML_print_list (struct GML_pair*, int);

/**
  * debugging
  */
void GML_printList(struct GML_pair*, int, ostream&);

/**
  * error
  */
char* GML_print_error (struct GML_stat*,const char* filename);

} // namespace graphlib

#endif // GML_PARSER_H

/* end of file */





