/* $Id: gml_scanner.cpp,v 1.1.1.1 2005/01/04 15:52:03 xtof Exp $ */
/***************************************************************************
                          gml_scanner.c  -  description
                             -------------------
    begin                :
    copyright            : (C) 1997 by Michael Himsolt
    email                : himsolt@fmi.uni-passau.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * Scanner for the GML - file format
 */

#ifndef _GML_SCANNER_CPP_
#define _GML_SCANNER_CPP_

#include <cstdlib>
#include <cassert>
#include <cstring>
#include <cctype>
#include <sstream>
#include "gml_scanner.h"


#include <list>
#include <string>
#include <typeinfo>

namespace { //moved from .cpp to .h by gil 090702 for GML_scanner() start
  //  #include "gml_utils.h"

  using namespace graphlib::graphlib_utils;
}

namespace graphlib {

/*
 * ISO8859-1 coding of chars >= 160
 */

string GML_table[] =
{
    "&nbsp;",     /* 160 */
    "&iexcl;",
    "&cent;",
    "&pound;",
    "&curren;",
    "&yen;",
    "&brvbar;",
    "&sect;",
    "&uml;",
    "&copy;",
    "&ordf;",     /* 170 */
    "&laquo;",
    "&not;",
    "&shy;",
    "&reg;",
    "&macr;",
    "&deg;",
    "&plusmn;",
    "&sup2;",
    "&sup3;",     /* 180 */
    "&acute;",
    "&micro;",
    "&para;",
    "&middot;",
    "&cedil;",
    "&sup1;",
    "&ordm;",
    "&raquo;",
    "&frac14;",
    "&frac12;",   
    "&frac34;",   /* 190 */
    "&iquest;",
    "&Agrave;",
    "&Aacute;",
    "&Acirc;",
    "&Atilde;",
    "&Auml;",
    "&Aring;",
    "&AElig;",
    "&Ccedil;",
    "&Egrave;",   /* 200 */
    "&Eacute;",
    "&Ecirc;",
    "&Euml;",
    "&Igrave;",
    "&Iacute;",
    "&Icirc;",
    "&Iuml;",
    "&ETH;",
    "&Ntilde;",
    "&Ograve;",   /* 210 */
    "&Oacute;",
    "&Ocirc;",
    "&Otilde;",
    "&Ouml;",
    "&times;",
    "&Oslash;",
    "&Ugrave;",
    "&Uacute;",
    "&Ucirc;",
    "&Uuml;",     /* 220 */
    "&Yacute;",
    "&THORN;",
    "&szlig;",
    "&agrave;",
    "&aacute;",
    "&acirc;",
    "&atilde;",
    "&auml;",
    "&aring;",
    "&aelig;",    /* 230 */
    "&ccedil;",
    "&egrave;",
    "&eacute;",
    "&ecirc;",
    "&euml;",
    "&igrave;",
    "&iacute;",
    "&icirc;",
    "&iuml;",
    "&eth;",      /* 240 */
    "&ntilde;",
    "&ograve;",
    "&oacute;",
    "&ocirc;",
    "&otilde;",
    "&ouml;",
    "&divide;",
    "&oslash;",
    "&ugrave;",
    "&uacute;",   /* 250 */
    "&ucirc;",
    "&uuml;",
    "&yacute;",
    "&thorn;",
    "&yuml;"
}; 

unsigned int GML_line;
unsigned int GML_column;


int GML_search_ISO (char* str, int len)
{
  int i;
  int ret = '&'; 

  // First check the extraordinary ones
  if (!strncmp (str, "&quot;", len))
    return 34;
  else if (!strncmp (str, "&amp;", len))
     return 38;
  else if (!strncmp (str, "&lt;", len))
     return 60;
  else if (!strncmp (str, "&gt;", len))
     return 62;
  	
  for (i = 0; i < 96; i++)
	  if (!strncmp (str, GML_table[i].c_str(), len))
	  {
	    ret = i + 160;
	    break;
	  }

  return ret;
}


void GML_init ()
{
    GML_line = 1;
    GML_column = 1;
}


string GML_error2string(struct GML_error err)
{
  stringstream message;
  switch (err.err_num)
  {
  case GML_OK:
    message << "OK"; break;
  case GML_UNEXPECTED:
    message << "character occured, where it makes no sense"; break;
  case GML_SYNTAX:
    message << "not a valid GML file"; break;
  case GML_PREMATURE_EOF:
    message << "EOF occured, where it was not expected"; break;
  case GML_TOO_MANY_DIGITS:
    message << "number of digits is limited to 1024"; break;
  case GML_OPEN_BRACKET:
    message << "too many opening brackets"; break;
  case GML_TOO_MANY_BRACKETS:
    message << "mismatch of brackets"; break;
  case GML_FILE_NOT_FOUND:
    message << "file not found"; break;
  case GML_NODE_ID_DOUBLE:
    message << "node id occured twice"; break;
  case GML_NO_GRAPH_FOUND:
    message << "no graph in the file"; break;
  case GML_NODE_NO_ID:
    message << "node without an id occured"; break;
  case GML_EDGE_NODE_UNKNOWN:
    message << "source or target node not in the graph"; break;
  case GML_EDGE_TWICE:
    message << "edge occures twice"; break;
  default:
    message << "unknown error"; break;
  }

  message << ": line " << err.line << ", column " << err.column;// << ends;
  return message.str();
}


/**
  * returns the next token in file. If an error occured it will be stored in 
  * GML_token.
  */

template <class X> struct GML_token GML_scanner(X source)
     // moved by gil from .cpp to .h 090702
{
    unsigned int cur_max_size = INITIAL_SIZE;
    static char buffer[INITIAL_SIZE];
    char* tmp = buffer;
    char* ret = tmp;
    struct GML_token token;
    int is_float = 0;
    unsigned int count = 0;
    int next;
    char ISO_buffer[8];
    int ISO_count;

    assert (source != NULL);
    
    int eof = seteof(source);

    // eliminate preceeding white spaces
    do
    {
      next = fgetc_ (source);
      GML_column++;

      if (next == '\n')
      {
        GML_line++;
        GML_column = 1;
     	}
        	
    } while (isspace (next) && next != eof);

    // reached EOF
    if (next == eof)
    {
      token.kind = GML_END;
      return token;
    }

    // floating point or integer
    else if (isdigit (next) || next == '.' || next == '+' || next == '-')
    {
      do
      {
        if (count == INITIAL_SIZE - 1)
        {
          token.value.err.err_num = GML_TOO_MANY_DIGITS;
          token.value.err.line = GML_line;
          token.value.err.column = GML_column + count;
          token.kind = GML_ERROR;
        	return token;
        }

        if (next == '.' || next == 'E')
          is_float = 1;

        buffer[count] = next;
        count++;
        next = fgetc_ (source);

      } while (!isspace(next) && next != ']' && next != eof);

      buffer[count] = 0;

      if (next == ']')
	  ungetc_ (next, source); 

      if (next == '\n')
      {
        GML_line++;
        GML_column = 1;
      }
      else
        GML_column += count;
        	
      if (is_float)
      {
        token.value.floating = atof (tmp);
        token.kind = GML_DOUBLE;
      }
      else
      {
        token.value.integer = atol (tmp);
        token.kind = GML_INT;
      }
      return token;
    }

    // key
    else if (isalpha (next) || next == '_')
    {
      do
      {
        if (count == cur_max_size - 1)
        {
          *tmp = 0;
        	tmp =  (char*) malloc(2 * cur_max_size * sizeof (char));
        	strcpy (tmp, ret);
        		
        	if (cur_max_size > INITIAL_SIZE)
        	  free (ret);

       		ret = tmp;
       		tmp += count;
       		cur_max_size *= 2;
       	}
        	
        *tmp++ = next;
        count++;
        next = fgetc_ (source);
      } while (isalnum (next) || next == '_');

      if (next == '\n')
      {
        GML_line++;
        GML_column = 1;
      }
      else
        GML_column += count;
        	
      if (next == '[')
        ungetc_ (next, source);

      else if (!isspace (next))
      {
        token.value.err.err_num = GML_UNEXPECTED;
        token.value.err.line = GML_line;
        token.value.err.column = GML_column + count;
        token.kind = GML_ERROR;
        	
        if (cur_max_size > INITIAL_SIZE)
          free (ret);

        return token;
      }

      *tmp = 0;
      token.kind = GML_KEY;
      token.value.str = (char*) malloc((count+1) * sizeof (char));
      strcpy (token.value.str, ret);
        	
      if (cur_max_size > INITIAL_SIZE)
        free (ret);

      return token;
    }

    //  comments, brackets and strings
    else
    {
      switch (next)
      {
      case '#':
        do
      		next = fgetc_ (source);
        while (next != '\n' && next != EOF);
        		
      	GML_line++;
      	GML_column = 1;
      	return GML_scanner (source);

      case '[':
        token.kind = GML_L_BRACKET;
      	return token;

      case ']':
        token.kind = GML_R_BRACKET;
      	return token;
        		
      case '"':
        next = fgetc_ (source);
      	GML_column++;

      	while (next != '"')
      	{
      		if (count >= cur_max_size - 8)
      		{
      		  *tmp = 0;
      		  tmp = (char*) malloc (2 * cur_max_size * sizeof(char));
      		  strcpy (tmp, ret);
        		
      		  if (cur_max_size > INITIAL_SIZE)
      		    free (ret);

      		  ret = tmp;
      		  tmp += count;
      		  cur_max_size *= 2;
      		}
        	
      		if (next == '&')
      		{
      		  ISO_count = 0;

      		  while (next != ';')
      		  {
      		    if (next == '"' || next == eof)
      		    {
      			    ungetc_ (next, source);
      			    ISO_count = 0;
      			    break;
      			  }
        			
        			if (ISO_count < 8)
        			{
        			  ISO_buffer[ISO_count] = next;
        			  ISO_count++;
        			}

        			next = fgetc_ (source);
        		}
          			
        		if (ISO_count == 8)
        			ISO_count = 0;

        		if (ISO_count)
        		{
        			ISO_buffer[ISO_count] = ';';
        			ISO_count++;
        			next = GML_search_ISO (ISO_buffer, ISO_count);
        			ISO_count = 0;
        		}
          		
        		else
        			next = '&';
        }
      	*tmp++ = next;
      	count++;
      	GML_column++;
        			
      	next = fgetc_ (source);

      	if (next == eof)
      	{
      	  token.value.err.err_num = GML_PREMATURE_EOF;
      		token.value.err.line = GML_line;
      		token.value.err.column = GML_column + count;
      		token.kind = GML_ERROR;
        		
      		if (cur_max_size > INITIAL_SIZE)
      		  free (ret);
        			
   		    return token;
     		}

     		if (next == '\n')
     	  {
      	  GML_line++;
      		GML_column = 1;
      	}
      }

      *tmp = 0;
      token.kind = GML_STRING;
      token.value.str = (char*) malloc((count+1) * sizeof (char));
      strcpy (token.value.str, ret);
        	
      if (cur_max_size > INITIAL_SIZE)
        free (ret);

      return token;
        	
    default:
      token.value.err.err_num = GML_UNEXPECTED;
      token.value.err.line = GML_line;
      token.value.err.column = GML_column;
      token.kind = GML_ERROR;
      return token;
    }		
  }
}

template struct GML_token GML_scanner<string*>(string*);
template struct GML_token GML_scanner<char**>(char**);//needed for template linking

	
} // namespace graphlib

#endif
/* end of file */
