/***************************************************************************
                          laplacianmatrix.cpp  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "laplacianmatrix.h"
#include "trace.h"

#include <iomanip>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
LaplacianMatrix::LaplacianMatrix() : laplacianM(0), sizeM(0) {;}

LaplacianMatrix::LaplacianMatrix(const Graph& g) : sizeM(g.numberNodes())
{
  if (sizeM == 0)
    THROW(runtime_error,
      "LaplacianMatrix::LaplacianMatrix(const Graph&) empty graph")

  // create Adjacency and Degree Matrix
  AdjacencyMatrix am(g);

  // make Laplacian
  makeLaplacian(am);

  nodes = am.nodes;
}

LaplacianMatrix::LaplacianMatrix(const AdjacencyMatrix& am) : sizeM(am.sizeM)
{
  if (! am.degreeM || !am.adjacencyM || sizeM==0)
    THROW(runtime_error,"LaplacianMatrix::LaplacianMatrix\
      (const AdjacencyMatrix*) AdjacencyMatrix==NULL")

  // make Laplacian
  makeLaplacian(am);

  nodes = am.nodes;
}

LaplacianMatrix::~LaplacianMatrix()
{
  if (laplacianM) delete[] laplacianM;
}

//---------------------------------------------------------------------------
// In-/Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os,const LaplacianMatrix& l)
{
  os << "\nLaplacian Matrix\n";

  os.precision(2);
  os.setf(ios::showpos);

  for (unsigned i=0; i!=l.sizeM; i++)
  {
    for (unsigned j=0; j!=l.sizeM; j++)
      os << l.laplacianM[l.sizeM*i+j] << "   ";
    os << endl;
  }

  return os;
}

void LaplacianMatrix::writeMathematica(ostream& os)
{
  os << "lapl = { { ";

  for (unsigned i=0; i!=sizeM; i++)
  {
    for (unsigned j=0; j!=sizeM; j++)
    {
      os << (int)laplacianM[sizeM*i+j];

      if (j!=sizeM-1)
        os << " , ";
    }
    if (i!=sizeM-1)
      os << " } , { ";
  }
  os << " } }; " << endl << endl;
}

//---------------------------------------------------------------------------
// Utilities
//---------------------------------------------------------------------------
void LaplacianMatrix::makeLaplacian(const AdjacencyMatrix& am)
{
  laplacianM = new double[sizeM*sizeM];
  fill(laplacianM,laplacianM+sizeM*sizeM,0);

  for (unsigned i=0; i!=sizeM*sizeM; i++)
    laplacianM[i] = am.degreeM[i] - am.adjacencyM[i];
}

} // namespace graphlib

/* end of file */
