/***************************************************************************
                          laplacianmatrix.h  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LAPLACIANMATRIX_H
#define LAPLACIANMATRIX_H

#include "adjacencymatrix.h"

#include <iostream>

namespace graphlib {

/**
  * @short Calculation of the laplacian matrix of a graph
  */

class LaplacianMatrix
{
  	//------------------------------------------------------ friend classe
  	friend class Eigenvector_Eigenvalue;
  	
  	//------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param os the output-stream
      * @param l Laplacian Matrix
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os,const LaplacianMatrix& l);

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Constructor. Creates empty instance.
      *
      * @param graph
      */
          LaplacianMatrix();

    /**
      * Constructor. Creates the Laplacian matrix of the <code>graph</code>.
      * <em>Please note:</em> <code>graph</code> may not by empty </em>,
      * otherwise a runtime_error will be thrown.
      *
      * @param graph
      */
	  LaplacianMatrix(const Graph& graph);
	
    /**
      * Constructor. Creates the Laplacian matrix of a graph with <code>am</code>
      * as its adjacency matrix.
      *
      * @param am ... the adjacency matrix of a graph
      */
    LaplacianMatrix(const AdjacencyMatrix& am);
	
	  /**
	    * Destructur. Deletes the Laplacian matrix and frees memory.
	    */
	  ~LaplacianMatrix();
	
  	//------------------------------------------------------ Output
  	/**
  	  * Writes the adjacency and degree matrix to <code>out</code>.
  	  * Format: mathematic
  	  * lapl = { { .. , .. } , { .. , .. } };
  	  *
  	  * @param ostream (default stdout)
  	  */
  	void writeMathematica(ostream&);
  	
  private:
    /**
      * CopyConstructor -> not implemented
      */
    LaplacianMatrix(const LaplacianMatrix&);

    /**
      * Assignement operator -> not implemented
      */
    LaplacianMatrix& operator=(const LaplacianMatrix&);

  protected:  	
  	//------------------------------------------------------ datas
	  /// the Laplacian matrix
	  double    *laplacianM;
	  /// number of nodes
	  unsigned   sizeM;
        /// map node <-> unsigned
        NodeNumberMap   nodes;
	
	  //------------------------------------------------------ Utitlites
	  /**
	    * Creates the Laplacian matrix.
	    *
	    * @param adjaceny matrix
	    */
	  void makeLaplacian(const AdjacencyMatrix&);
	
	
}; // class LaplacianMatrix

} // namespace graphlib
#endif //LAPLACIANMATRIX_H

/* end of file */
