/***************************************************************************
                          logfile.h  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Petra Gleiss
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOGFILE_H
#define LOGFILE_H

#include <fstream>
#include <string>

#include "graphlib.h"

using namespace std;

namespace graphlib {

/**
  * class for debugging calculcations and storing provisional results
  */

class LogFile
{
	public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Generates an NULL-pointer ofstream
      */
		LogFile(){out=NULL; }

    /**
      * Closes ofstream and deletes it
      */
		~LogFile();

		/**
		  * Initialize a new ofstream binding on the file <code>fname</code>
		  *
		  * @param <code>fname</code> name of logfile
		  */
		void init(string fname);
		
		/**
		  * Tries to bind the ofstream on the existing file <code>fname</code>.
		  *
		  * @param <code>fname</code> name of logfile
		  */
		void reopen(string fname);

  	//------------------------------------------------------ Methods
		/**
		  * Writes the message into the logfile <code>fname</code>.
		  * <code>level</code> defines the debug-level.
		  *
		  * @param message of the statement
		  */
		void logging(string message);
		
	  /**
	    * Returns ostream&
	    */
	  ostream& os() { return *out;}
	
	  /**
	    * Returns true if initialized.
	    */
	  bool isInit() const { return (out==NULL) ? false : true;}

	protected:
  	//------------------------------------------------------ ofstrem
		ofstream *out;

}; // class LogFile

} // namespace graphlib

	extern graphlib::LogFile logfile;

  //------------------------------------------------------ some macros
	#define INIT_LOGFILE(fname) logfile.init(fname);
	
	#ifdef GRAPHLIB_DEBUG
	  #if GRAPHLIB_DEBUG==1
	    #define LOG1(msg) logfile.logging(msg);
    	#define LOG2(msg) {;}
    	#define LOG3(msg) {;}
    	#define LOG4(msg) {;}
    	#define LOG5(msg) {;}
	
	  #elif GRAPHLIB_DEBUG==2
	    #define LOG1(msg) logfile.logging(msg);
      #define LOG2(msg) logfile.logging(msg);
    	#define LOG3(msg) {;}
    	#define LOG4(msg) {;}
    	#define LOG5(msg) {;}

    #elif GRAPHLIB_DEBUG==3
	    #define LOG1(msg) logfile.logging(msg);
      #define LOG2(msg) logfile.logging(msg);
      #define LOG3(msg) logfile.logging(msg);
    	#define LOG4(msg) {;}
    	#define LOG5(msg) {;}

    #elif GRAPHLIB_DEBUG==4
	    #define LOG1(msg) logfile.logging(msg);
      #define LOG2(msg) logfile.logging(msg);
      #define LOG3(msg) logfile.logging(msg);
      #define LOG4(msg) logfile.logging(msg);
    	#define LOG5(msg) {;}

    #elif GRAPHLIB_DEBUG>4
	    #define LOG1(msg) logfile.logging(msg);
      #define LOG2(msg) logfile.logging(msg);
      #define LOG3(msg) logfile.logging(msg);
      #define LOG4(msg) logfile.logging(msg);
      #define LOG5(msg) logfile.logging(msg);

    #endif // GRAPHLIB_DEBUG

  #else
  	#define LOG1(msg) {;}
  	#define LOG2(msg) {;}
  	#define LOG3(msg) {;}
  	#define LOG4(msg) {;}
  	#define LOG5(msg) {;}

  #endif //GRAPHLIB_DEBUG

#endif // LogFile_H

/* end of file */
