/***************************************************************************
                          node.cpp  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "node.h"
#include "nodedata.h"
#include "coordinates.h"
#include "edgedata.h"
#include "trace.h"

#include <algorithm>
#include <sstream>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Node::Node() : data(NULL)
{
}

//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
bool Node::operator==(const Node& n) const
{
  return (data == n.data);
}
bool Node::operator!=(const Node& n) const
{
  return (data != n.data);
}
bool Node::operator<(const Node& n) const
{
  return (data->id < n.data->id);
}

//---------------------------------------------------------------------------
// Input / Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os, const Node& n)
{
  constEdgeIter it;
  os << "["  << n.id() << "]";
  for (it=n.data->edges.begin(); it!=n.data->edges.end();it++)
    os << "       ["<< *it << "]";

  return os;
}

//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
unsigned Node::id() const
{
  return data->id;
}

constNodeIter Node::pos() const
{
  return data->pos;
}

unsigned Node::degree() const
{
  return data->degree;
}

string Node::label() const
{
  return data->label;
}

Coordinates Node::coord() const
{
  return data->coord;
}

const Node& Node::opposite(const Edge& e) const
{
  if (!e.data)
      THROW(logic_error,"Node::opposite(const Edge&) edge has no data!\n")

  Node& s = *(e.data->nodes.begin());
  if (*this == s)   // if this node == source of e
    return *(++(e.data->nodes.begin()));
  else              // else
    return s;
}

bool Node::isNULL() const
{
  return (data == NULL ? true : false);
}
//---------------------------------------------------------------------------
// Set Funtions
//---------------------------------------------------------------------------

constNodeIter Node::label(string label)
{
  data->label = label;
  return data->pos;
}
constNodeIter Node::coord(Coordinates coord)
{
  data->coord = coord;
  return data->pos;
}

//---------------------------------------------------------------------------
// Iterators
//---------------------------------------------------------------------------
constEdgeIter Node::adjBegin() const
{
  return data->edges.begin();
}

constEdgeIter Node::adjEnd() const
{
  return data->edges.end();
}


} // namespace graphlib

/* end of file */
