/***************************************************************************
                          dot2gml.cpp  -  description
                             -------------------
    begin                : Sat Jul 28 2001
    copyright            : (C) 2001 by Petra Gleiss
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "smi2gml.h"
#include "utilities.h"
#include "trace.h"
#include "stdio.h"

#include <map>
#include <string>
using namespace std;

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Smi2Gml::~Smi2Gml()
{
  if (in != NULL)
  {
    in->close();
    delete in;
  }
}

void Smi2Gml::init(string fname)
{
  if (fname == "-") {
    in = (ifstream *)&cin;
  }

  in = new ifstream(fname.c_str(), ios::in);

  if (in->bad() || in->fail())
    throw logic_error(("Can't open smi file "+fname+".").c_str());
}

void Smi2Gml::init_smi(string smi)
{
  structure = smi; 
}

//---------------------------------------------------------------------------
// Methods
//---------------------------------------------------------------------------
void Smi2Gml::smi2gml(Graph& g)
{
  g.clear();

  // reads file -> string
  if (!structure.size())
    readStructure();

  // structure -> graph
  makeGraph(g);

  // minCoord = -inf maxCoord = inf
  //g.resetMinMaxCoord();

  // set name is 
  g.name = structure; 
}

int Smi2Gml::addH(Graph& g)
{
  int nbH = 0;
  map<Node, unsigned> impH;

  constNodeIter it = g.nodesBegin();
  constNodeIter end = g.nodesEnd();
  while (it != end)
  {
    int deg = 0; //- it->degree();
    constEdgeIter itE = it->adjBegin();
    constEdgeIter endE = it->adjEnd();
    while (itE != endE)
    {
      if (itE->label() == "")
      {
	deg -= 2;
	itE++;
	continue;
      }
      switch (*(itE->label().begin()))
      {
      case '-' :
	deg -= 2;
	break;
      case '=' :
	deg -= 4;
	break;
      case ':' :
	deg -= 3;
	break;
      case '#' :
	deg -= 6;
	break;
      }
      itE++;
    }

    string::iterator label_it = it->label().begin();
    switch ( *label_it != '[' ? *label_it : *(++label_it) )
    {
    case 'C' :
      deg += 8;
      break;
    case 'N' :
      deg += 6;
      break;
    case 'O' :
      deg += 4;
      break;
    default:
      deg = 0;
      break;
    }

    if (label_it != it->label().begin())
    {
      if ( *(++label_it) == '+') 
        deg -= 2 * (*(++label_it) - '0');
      if ( *(label_it) == '-')
        deg += 2 * (*(++label_it) - '0');       
    }
    
    if (deg)
      impH[*it] = (deg >> 1);
    it++;
  }
  
  map<Node, unsigned>::iterator pair = impH.begin();
  map<Node, unsigned>::iterator pend = impH.end();
  while (pair != pend)
  {
    for (unsigned i = 0; i < pair->second; i++)
    {
      NodeIter H = g.newNode();
      H->label("H");
      g.newEdge(*H, pair->first, "-");
      nbH++;
    }
    pair++;
  }
  return nbH;
}

  

void Smi2Gml::readStructure()
{

  if (in == 0) 
    throw logic_error("No file opened!");


  getline(*in,structure); 
  
} // namespace graphlib

void Smi2Gml::makeGraph(Graph& g)
{
  unsigned i=0, length=structure.length(); //current position, length
  vector<int> rund; // stack of branching points (they precede 'rund' brackets)
  int now = -1; int prev = -1; // ids of current and previous nodes
  NodeIter nI;

  //determine highest ring closure id
  nb_cyc = 20;
#if 0
  // fix "find highest ringclosure index"
  {
    char* str = const_cast<char*>(structure.c_str());
    unsigned tmp = 0, len = 0;
    while (sscanf(str,"%*s%d%n",&tmp, &len))while (sscanf(str,"%*[^0-9]%d%n",&tmp, &len) > 0)
    {
      if (tmp > nb_cyc)
	nb_cyc = tmp;
      str += len;
    }
  }
#endif
  //printf("HAHA%d",nb_cyc);


  string edge_label = "", node_label = ""; 

  for (i=0; i!=length; i++)
  {
    // this node
    //now = g.newNode();

    /*if (i!=0 && ( structure[i] == '=' || structure[i] == '-' ||
		  structure[i] == ':' || structure[i] == '#' ) ) {
       edge_label = structure[i];
       continue;
       }*/

    // if we run into an atom coded by a sinle letter (upcase or lowcase)
    if ( (structure[i] >= 'a' && structure[i] <= 'z') || 
	 (structure[i] >= 'A' && structure[i] <= 'Z') ) {
      nI = g.newNode();  // add node
      now = nI->id();    // make his id the current one
      nI->label(string(1,structure[i])); // set his label to the letter
      // backbone: edge(prev,now) is added unless start of smiles or fragment
      if (i != 0 && structure[i-1] != '.') {
	g.newEdge(prev, now, (edge_label == "") ? "-" : edge_label);
	edge_label = "";
      }
      prev = now; // current node is now previous node
      // if we run into a ring closure coded by a single digit
    } else if (structure[i] >= '0' && structure[i] <= '9') {
      // handle by getCycle
      getCycle(g, structure[i] - '0', edge_label, prev);
    } else {
      // other possibilities
    switch(structure[i])
    {
      // beginning of a fragment / error if it comes after an edge symbol
      case '.' :
    	if (edge_label != "")
    	  THROW(runtime_error,"edge with no target - "+Ttos(i))	
        prev = -1; now = -1;
	edge_label = "";
	break;

      // edge symbol
      case '=' : case '#' : case ':' : case '-' :
	edge_label = structure[i];
	break;
      
      // if we run in to an atom coded by more than one letter
      case '[':
	node_label = "";
	// set label to all until closing ]
	for (i++; structure[i] != ']' ; i++) node_label += structure[i];
	nI = g.newNode();
	now = nI->id();
	nI->label(node_label);
	node_label = "";
	// backbone: edge(prev,now)
	if (i != 0 && prev != -1) {
	  g.newEdge(prev,now, (edge_label == "") ? "-" : edge_label);
	  edge_label = "";
	}	
	prev = now;
        break;

	// if we run into a branching
      case '(':
	if (i == 0 || prev == -1)
	  THROW(runtime_error,"Can't  start with '(' - "+Ttos(i))
	    // push current node into stack
	rund.push_back(prev);
	break;
	
	// if we run into an end of branch
      case ')':
	if (rund.empty())
	  THROW(runtime_error,"unbalanced ()-brackets - "+Ttos(i));
	// set current node to top of stack
	prev = rund.back();
	edge_label = "";
	rund.pop_back();    	
	break;
	
	// if we run into a ring closure coded by two digits
      case '%' :
	if (! ( (structure[i+1] >= '0' && structure[i+1] <= '9') &&
		(structure[i+2] >= '0' && structure[i+2] <= '9') ) ) 
	  THROW(runtime_error,"invalid ring closure index - "+Ttos(i)); 
	// handle by getCycle()
	getCycle(g, (structure[i++]-'0')*10 + (structure[i++]-'0'), edge_label, prev);
	break;

    	
      // may not happen !!!
      default: 
        THROW(runtime_error,"undefined symbol nr "+Ttos(i)+": "+ structure[i])
    } }



    // prev <- this node
    //prev = now;
  }

  if (!rund.empty())
    THROW(runtime_error,"unbalanced ()-brackets: open")

}

void Smi2Gml::getCycle(Graph& g, int i, string& edge, int prev) {
  static vector<string> cycleEdges(nb_cyc+1, string(""));
  static map< int , int > cycleCondPoints;
  //static const Node null;

  if (cycleCondPoints.count(i)) {
      g.newEdge(prev, cycleCondPoints[i], (edge != "") ? edge : cycleEdges[i]);
      edge = "";
      cycleEdges[i] = "";
      cycleCondPoints.erase(i);
    } 
   else {
     //cycleEdges.reserve(i+1); cycleEdges[i].reserve(1); 
     cycleEdges[i] = (edge != "") ? edge : "-"; //""; 
     edge = "";
     cycleCondPoints[i] = prev;
    }
}

} //namespace graphlib

/* end of file */
