/***************************************************************************
                          utilities.h  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef UTILITIES_H
#define UTILITIES_H

#include <string>
#include <iomanip>
#include <sstream>
#include <string>
#include <ctime>
#include <iterator>

namespace graphlib {

  /**
   *     * Output-operator.
   *         */
  template<class T> inline
  ostream& operator<<(ostream& os, const list<T>& vc)
  {
    copy(vc.begin(),vc.end(),ostream_iterator<T>(os," "));
    return os;
  }
	  
	
  /**
    * Returns the smaller one of <code>a</code> and
    * <code>b</code>.
    */
  template<class T> inline
  const T& minimum(const T& a, const T& b)
  {
    if (a < b)
      return a;
    else
      return b;
  }

  /**
    * Returns the <code>arg</code> as string.
    */
  template <typename T> inline
  string Ttos(T arg)
  {
    ostringstream buffer;
    buffer << arg;// << ends;
    return buffer.str();
  }

  /**
    * Formates <code>arg</code> to: 1.xxxxxxxxxxxxxxxx*^Exponent.
    * (for writeMathematica)
    */
  template <typename T> inline
  string toMathematica(T arg)
  {
    ostringstream buffer;
    string zahl;

    buffer.setf(ios::scientific);
    buffer.precision(16);
    buffer << arg;// << ends;
    zahl = buffer.str();
    //buffer.freeze(0);
    zahl.replace(zahl.find("e"),1,"*^");
    return zahl;
  }

  /**
    * Returns the adsolute difference between <code>a</code>
    * and <code>b</code>.
    */
  template <typename T> inline
  T delta(const T& a, const T& b)
  {
    if (a < b)
      return (b-a);
    else
      return (a-b);
  }

  /**
    * Returns the localtime as string.
    * Format: e.g. Mon Mar  8 11:01:44 2004
    */
  inline string timeStamp()
  {
    time_t datum;
    localtime(&datum);
    return string(ctime(&datum));
  }


} // namespace graphlib
#endif // UTILITIES_H

/* end of file */
