#include "graph.h"
#include <iostream>
#include <cstdlib>
#include <unistd.h>
#include "trace.h"
#include "logfile.h"
#include "gml2smi.h"
#include "smi2gml.h"
#include <vecmath.h>
#include "eht.h"
#include "springEmbedder.h"
#include <string>
#include <list>

// logging exceptions and tracing
graphlib::Trace trace;
// logging debug messages
graphlib::LogFile logfile;

using namespace graphlib;

void usage()
{
  cerr << "Usage: getE [-acdeEso] <molfile, has to be last argument>" << endl;
  cerr << "a_ll; c_harges; d_ipole; e_nergy; E_igenemebed; s_pectrum; o_rbitals;" << endl;
  cerr << "p_arameters <file>" << endl;
  cerr << "S_olvation enthalpy <relative permittivity>" << endl; 
  exit(0);
}

class Tmp : public Eht
{
public:
  Tmp(const Graph& g,
      string fname = "",
      unsigned nO = 0) : Eht(g,fname,nO)
  {
  ostream& os = cout;
  os.precision(2);
  os << "lapl = { { ";

  for (unsigned i=0; i!=sizeM; i++)
  {
    for (unsigned j=0; j!=sizeM; j++)
    {
      os << overlapM[sizeM*i+j];

      if (j!=sizeM-1)
        os << " , ";
    }
    if (i!=sizeM-1)
      os << " } , { ";
  }
  os << " } }; " << endl << endl;

  }
};

int main(int argc, char *argv[]) {


  // initialize trace
  INIT_TRACE("trace.graphlibtest")
    
  // initialize logfile
  INIT_LOGFILE("log.graphlibtest")  

  int options;
  bool energy = false;
  bool dipole = false;
  bool charges = false;
  bool eigen_embed = false;
  bool spectrum = false;
  bool orbs = false;
  char* params = "eht.par";
  double solv = 0.;

  char* arg1 = argv[argc-1];
  
  while ((options = getopt(argc, argv, "acdeEsS:op:")) != EOF)    
  {
    switch (options)
    {
      case 'a': 
	charges = true;
	dipole = true;
	energy = true;
	spectrum = true;
        break;

      case 'c': 
	charges = true;
        break;

      case 'd': 
	dipole = true;
        break;

      case 'e': 
	energy = true;
        break;

      case 'E': 
	eigen_embed = true;
        break;

     case 's': 
	spectrum = true;
        break;

     case 'S': 
	solv = atof(optarg);
        break;

     case 'o': 
	orbs = true;
        break;

     case 'p': 
	params = optarg;
        break;
			
      default: usage();
    }
  }

  try
  {
    // initialize EHT parameters
    Smi2Gml s;
    Graph mol; 
    cerr << "argv[argc-1]=" << arg1 << endl;
    s.init_smi(arg1); 
    s.smi2gml(mol);
    
    //s.addH(mol);
    if (orbs)
      cout << (HamiltonMatrix(mol,params,0)).graph;
    Eht X(mol,params,0);
    if (energy)
      cout << X;
    if (spectrum)
      cout << *((Eigenvector_Eigenvalue*)&X);
    if (charges)
      cout << X.printCharges();
    //cout << X.graph;
    if (dipole)
      cout << X.printDipoleMmt() << endl;
    if (solv)
      cout << X.printSolvE(solv) << endl;
      //cout << mol.numberNodes() << "\t" << X.printSolvE(1.5+solv)-X.printSolvE(1.5) << endl;
    if (eigen_embed)
    {
      D_capEmbedder(mol, X.hyb,X.charge,true);
      double sizex0,sizey0,sizez0;
      // Compute bounds
      ComputeBounds(mol,sizex0,sizey0, sizez0);
      for (constNodeIter v = mol.nodesBegin(); v != mol.nodesEnd(); v++)
      {Coordinates p = v->coord();
      p.x *= 100/sizex0;
      p.y *= 100/sizey0;
      p.z *= 100/sizez0;
      mol.findNode(v->id())->coord(p);
      }
      cout << mol;
    }
  }
  catch ( runtime_error& e )
  {
    std::cerr << "Exception was caught:" << e.what() << "\n";
  }  
  catch ( logic_error& e )
  {
    std::cerr << "Exception was caught:" << e.what() << "\n";
  }  


}

