/***************************************************************************
                            eht.h  -  description
                             -------------------
    begin                : Wed March 27 2002
    copyright            : (C) 2002 by Gil Benkoe
    email                : gil@tbi.univie.ac.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _EHT_H_
#define _EHT_H_

#include "graph.h"
#include "hamiltonmatrix.h"
#include "eigenvector_eigenvalue.h"
#include "eigen.h"
#include "eigen2.h"
#include <algorithm>
#include "cyclegraph.h"
#include "springEmbedder.h"

using namespace graphlib;

class Eht : public HamiltonMatrix,
	    public Eigenvector_Eigenvalue
{
 public:

  Eht(const Graph& g, 
      string fname = "", 
      unsigned nO = 0) : HamiltonMatrix(g, fname, nO),
                         Eigenvector_Eigenvalue(this, loew)
  {
    initCharges(origGraph);
   
    energy = 0;

    list<EigenPair*>::iterator it = eigenpairs.begin();
    list<EigenPair*>::iterator end = eigenpairs.end();

    if (isnan( (*it)->eigenValue() ))
	THROW(runtime_error,"Eht::Eht() orthogonalization failed" +origGraph.name)

    double *v;
    int k = eNb;

    NodeNumberMap::const_iterator node;

    for (int j=0;it != end;j++,it++) 
    {
      v = (*it)->eigenVector();
      node=this->HamiltonMatrix::nodes.begin();
      for(unsigned i=0; i<sizeM;i++,node++) 
      {  
	if (k > 1) charge[origin[node->first]]
		       -= v[node->second] * v[node->second] * 2;
	else if (k == 1) charge[origin[node->first]]
			     -= v[node->second] * v[node->second];
      }

      if (k > 1)
      {
	energy += (*it)->eigenValue() * 2; 
	k -= 2;
      } else if (k == 1) 
      {
	energy += (*it)->eigenValue(); 
	k = 0;
      }
    }

    constNodeIter itN = origGraph.nodesBegin();
    constNodeIter endN = origGraph.nodesEnd();
    while (itN != endN) 
      energy -= Hf[string(1,itN++->label()[0])];

    energy /= 0.04336;

    coreRepulsion();
    energy *= 0.5;
    energy += rep;
  }

  	//------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param os the output-stream
      * @param a AdjacencMatrix
      * @return the output-stream
      */
      friend ostream& operator<<(ostream& os,const Eht& l);


 public:
  //map original Node <-> charge
  map<Node,double> charge; 
  double energy, rep;

  void coreRepulsion()
  {
    rep = 0;

    constEdgeIter itE = origGraph.edgesBegin();
    constEdgeIter endE = origGraph.edgesEnd();
    while (itE != endE)
    {
      const Node& source = itE->source();
      const Node& target = itE++->target();
      double Zs = Z(source.label());
      double Zt = Z(target.label());
      bool HX = ((source.label()[0] != 'H') && (target.label()[0] != 'H'));
      rep += (HX ? rep_X_X : rep_H_X) * Zs * Zt; 
      rep += 14.4 * 0.5 * ( ( (HX ? V_X : V_H) * Zs * charge[target] ) +
			    ( (HX ? V_X : V_H) * Zt * charge[source] ) );
    }
    //  / 2.4 * overlap[pair<string,string>(itE->source().label() + hyb[itE->source()],...
//#if 0
    constNodeIter itN = origGraph.nodesBegin();
    constNodeIter endN = origGraph.nodesEnd();
    while (itN != endN)
    {
      constEdgeIter itE = itN->adjBegin();
      constEdgeIter itE2;
      constEdgeIter endE = itN->adjEnd();
      while (itE != endE)
      {
	const Node& source = itN->opposite(*itE++);
	itE2 = itE; 
	while (itE2 != endE)
	{
	  const Node& target = itN->opposite(*itE2++);
	  double Zs = Z(source.label());
	  double Zt = Z(target.label());
	  bool HX = ((source.label() != "H") && (target.label() != "H"));
	  rep += (HX ? rep_X_X : rep_H_X) * Zs * Zt / 3; 
	  rep += 14.4 * 0.5 * ( ( (HX ? V_X : V_H) * Zs * charge[target] ) +
				( (HX ? V_X : V_H) * Zt * charge[source] ) );
	}
      }
      itN++;
    }
//#endif
    rep *= 23.061;

/*
    constEdgeIter itE = origGraph.edgesBegin();
    constEdgeIter endE = origGraph.edgesEnd();
    while (itE != endE)
    {
      const Node& source = itE->source();
      const Node& target = itE++->target();
      rep += ( (source.label() != "H" ? 1.7 : 1.0) * 
               (target.label() != "H" ? 1.7 : 1.0) ) *
        (1 + (charge[source]/Z(source.label()) )) *
        (1 + (charge[target]/Z(target.label()) )) ;
    }
    //  / 2.4 * overlap[pair<string,string>(itE->source().label() + hyb[itE->source()],...
    rep /= 1.3/0.04336;
*/

  }    

  void initCharges(const Graph& g)
  {

    constNodeIter it = g.nodesBegin();
    constNodeIter end = g.nodesEnd();
    while (it != end)
    {
      charge[*it] = Z(it->label());
      it++;
    }

  }

  double Z(string label)
  {
      switch( *label.begin() )
      {
      case 'H' :
	return 1.;
	break;
      case 'C' :
	return 4.;
	break;
      case 'N' :
	return 5.;
	break;
      case 'O' :
	return 6.;
	break;
      case 'P' :
        return 15.;
        break;
      case 'S' :
        return 16.;
        break;

      }
      return 4.; //consider as C
  }

  Graph printMO(constEigenPairIter MO)
  {
    Graph MOg = origGraph;
    NodeNumberMap::const_iterator node=this->HamiltonMatrix::nodes.begin();
    for(unsigned i=0; i<sizeM;i++,node++) 
      if (fabs((*MO)->eigenVector()[node->second]) > 0.05)
	MOg.findNode(origin[node->first].id())
	  ->label( MOg.findNode(origin[node->first].id())->label()
		   + " " + node->first.label()
		   + " " + Ttos( (*MO)->eigenVector()[node->second] ) ); 
    return MOg;
  }

  Graph printMO(int n)
  {
    constEigenPairIter MO = eigenpairs.begin();
    advance(MO, n);
    return printMO(MO);
  }
    

  
  Graph printCharges()
  {
    Graph Cg = origGraph;
    constNodeIter it = origGraph.nodesBegin();
    constNodeIter end = origGraph.nodesEnd();

    while (it != end)
    {
      Cg.findNode(it->id())->label(it->label() + " " + Ttos(charge[*it]));
      it++;
    }

    return Cg;
  }
  
  
  //algebragraph/component
  double printDipoleMmt()
  { 
    Graph Cg = origGraph;
    return(D_capEmbedder(Cg, hyb,charge,false));
    springEmbedder(Cg,100,100,100,true);
    Coordinates dipole_vec(0,0,0);
    constNodeIter it = Cg.nodesBegin(); 
    constNodeIter it2 = Cg.nodesBegin(); it2++;
    constNodeIter end = Cg.nodesEnd();

    double len = sqrt( 
	 (it->coord().x-it2->coord().x)*(it->coord().x-it2->coord().x) +
	 (it->coord().y-it2->coord().y)*(it->coord().y-it2->coord().y) +
	 (it->coord().z-it2->coord().z)*(it->coord().z-it2->coord().z) );

    for (; it != end; it++)
    {
      dipole_vec.x += it->coord().x * charge[*it];
      dipole_vec.y += it->coord().y * charge[*it];
      dipole_vec.z += it->coord().z * charge[*it];
    }

    dipole_vec.x /= len;
    dipole_vec.y /= len;
    dipole_vec.z /= len;

    double dipole = sqrt( dipole_vec.x*dipole_vec.x +
			  dipole_vec.y*dipole_vec.y +
			  dipole_vec.z*dipole_vec.z );

    return dipole;
  }
      
  double printSolvE(double eps_0)
  {
    CycleGraph g = CycleGraph::Graph(origGraph);

    NodePathMap2 nodeMap;
    NodeSet nodeSet;
    constNodeIter r,i;
    //Component g(this, edgesBegin(), edgesEnd());
    unsigned meanDist = 0;
    // wiener index = r^3 = meanDist/4
    for (r = g.nodesBegin(); r != g.nodesEnd(); r++)
    {
      g.initNodeMap(*r,g,nodeMap,nodeSet);
      g.dijkstra(*r,nodeMap,nodeSet,true);
      for (i = g.nodesBegin(); i != g.nodesEnd(); i++)
      {
        if (i == r || i->label() == "H" || r->label() == "H")//why not with 'H'
          continue;
        meanDist += nodeMap[*i].distance;
      }
    }

    double mu = printDipoleMmt();
    //kirkwood, relative permittivity
    return ( (eps_0-1)/(2*eps_0+1) ) * mu*mu/(meanDist*.25*0.01);

      /*
    //refined (polarisable)
    //polarizability
    double alpha = 0;
    for (constNodeIter it = origGraph.begin();
	 it != origGraph.end(); it++)
      if (it->label().begin() == 'H')
      {
	alpha += 0.3;
      } else {
	alpha += 1.3;
      }
    //from Miller & Savchik, 1990
    alpha = (4/eNb)*(alpha*alpha);
    // E= ... / ( 1 - (2alpha/a^3 * eps-1/2eps+1))
    */
  }

  constEigenPairIter HOMO() 
  { 
    constEigenPairIter MO = eigenpairs.begin();
    advance(MO, (eNb-1) >> 1);
    return MO;
  }    

  constEigenPairIter LUMO()
  { 
    constEigenPairIter MO = eigenpairs.begin();
    advance(MO, ((eNb-1) >> 1) +1);
    return MO;
  }    

  double scaledEnergy(string fname = "")
  {
    if (fname == "" && _overlap.size() != _overlap_alt.size())
    {
      cerr << "Can't scale" << endl;
      return 0;
    }
    Eht X(origGraph, fname, (nO & 1) + 2);
    return (X.energy + scale*(energy-X.energy));
  }

};


//---------------------------------------------------------------------------
// In-/Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os,const Eht& l)
{
  os.precision(6);
  os.setf(ios::showpos);


  os << "EHMO energy (kcal/mol): " << l.energy-l.rep << endl;
  os << "Core repulsion: " << l.rep << endl;
  os << "Total: " << l.energy << endl;

  return os;
}


#endif
