/*  eigen.cpp aus CCMATH mathematics library source code.
 *
 *  Copyright (C)  2000   Daniel A. Atkinson    All rights reserved.
 *  This code may be redistributed under the terms of the GNU library
 *  public license (LGPL). ( See the lgpl.license file for details.)
 * ------------------------------------------------------------------------
 */

#include "eigen2.h"
#include "eigen.h"

#include <cstdio>
#include <cstdlib>
#include <cmath>

namespace graphlib {

// From the website of Steven Hai Jiang at CECS  Univ. of Missouri-Columbia

/* This is highly distributed program to implement Jacobi decomposition 
   for matrix eigen vector computation.
*/
/*
a: array nxn, double, when return diagonal save the n eigenvalues 
n: integer, the level of the array
v: eigen vectors, nxn, No. i column is the No. i eigen vector corresponding
	to eigenvalue a(i,i)
eps:  double real, to control the resolution of demand
jt:   integer, control maximum of iterations
*/

//#include <stdio.h>
//#include <math.h>

int eejcb(double* a, int n, double* v, double eps, int jt)
  //int n, jt;
  //double a[], v[], eps;
{
int i, j, p, q, u, w, t, s, l;
double fm, cn, sn, omega, x, y, d;

l=1;
for(i=0;i<n;i++)
	{
	v[i*n+i]=1.0;
	for(j=0;j<n;j++)
		if(i!=j) v[i*n+j]=0.0;
	}

/*
while(l==1)
*/
while(l<=jt)
	{
	fm=0.0;
	for(i=0;i<n;i++)
	for(j=0;j<n;j++)
		{
		d=fabs(a[i*n+j]);
		if((i!=j)&&(d>fm))
			{fm=d;p=i;q=j;}
		}
	if(fm<eps) return(1);
	if(l>jt) return(-1);
	l++;
	u=p*n+q; w=p*n+p; t=q*n+p; s=q*n+q;
	x=-a[u]; y=(a[s]-a[w])/2.0;
	omega=x/(double)(sqrt(x*x+y*y));
	if(y<0.0) omega=-omega;
	sn=1.0+(double)(sqrt(1.0-omega*omega));
	sn=omega/(double)(sqrt(2.0*sn));
	cn=(double)(sqrtf(1.0-sn*sn));
	fm=a[w];
	a[w]=fm*cn*cn+a[s]*sn*sn+a[u]*omega;
	a[s]=fm*sn*sn+a[s]*cn*cn-a[u]*omega;
	a[u]=0.0; a[t]=0.0;
	for(j=0;j<n;j++)
		if((j!=p)&&(j!=q))
			{
			u=p*n+j; w=q*n+j;
			fm=a[u];
			a[u]=fm*cn+a[w]*sn;
			a[w]=-fm*sn+a[w]*cn;
			}
	for(i=0;i<n;i++)
		if((i!=p)&&(i!=q))
			{
			u=i*n+p; w=i*n+q;
			fm=a[u];
			a[u]=fm*cn+a[w]*sn;
			a[w]=-fm*sn+a[w]*cn;
			}
	for(i=0;i<n;i++)
		{
		u=i*n+p; w=i*n+q;
		fm=v[u];
		v[u]=fm*cn+v[w]*sn;
		v[w]=-fm*sn+v[w]*cn;
		}
	}
return(1);
}

int psinv(double *v,int n)
{ double z,*p,*q,*r,*s,*t; int j,k;
  for(j=0,p=v; j<n ;++j,p+=n+1){
    for(q=v+j*n; q<p ;++q) *p-= *q* *q;
    if(*p<=0.) return -1;
    *p=sqrt(*p);
    for(k=j+1,q=p+n; k<n ;++k,q+=n){
      for(r=v+j*n,s=v+k*n,z=0.; r<p ;) z+= *r++ * *s++;
      *q-=z; *q/= *p;
     }
   }
  trnm(v,n);
  for(j=0,p=v; j<n ;++j,p+=n+1){ *p=1./ *p;
    for(q=v+j,t=v; q<p ;t+=n+1,q+=n){
      for(s=q,r=t,z=0.; s<p ;s+=n) z-= *s * *r++;
      *q=z* *p; }
   }
  for(j=0,p=v; j<n ;++j,p+=n+1){
    for(q=v+j,t=p-j; q<=p ;q+=n){
      for(k=j,r=p,s=q,z=0.; k<n ;++k) z += *r++ * *s++;
      *t++ =(*q=z); }
   }
  return 0;
}

int choldc(double *c,double *a,int n)
{ double sum; int i,j,k;
  for(i=1; i<=n ;i++){
    for(j=i; j<=n ;j++){
      for(sum= *(a+(i-1)*n+j-1),k=i-1; k>=1 ;k--) sum -= *(a+(i-1)*n+k-1) * *(a+(j-1)*n+k-1);
      if(i==j){
	if(sum<=0.) return -1;
	*(c+(i-1)*n+i-1) = sqrt(sum);
      }else{ *(c+(j-1)*n+i-1) = sum/ *(c+(i-1)*n+i-1); }
    }
  }
  return 0;
}

void vnorm(double *v,int n)
{ int i; double sum = 0.; 
  for(i=0; i<n ;i++) sum += v[i] * v[i];
  sum=sqrt(sum);
  for(i=0; i<n ;i++,v++) *v /= sum;
}

int minv(double *a,int n)
{ int lc,*le; double s,t,tq=0.,zr=1.e-15;
  double *pa,*pd,*ps,*p,*q,*q0;
  int i,j,k,m;
  le=(int *)malloc(n*sizeof(int));
  q0=(double *)malloc(n*sizeof(double));
  for(j=0,pa=pd=a; j<n ;++j,++pa,pd+=n+1){
    if(j>0){
      for(i=0,q=q0,p=pa; i<n ;++i,p+=n) *q++ = *p;
      for(i=1; i<n ;++i){ lc=i<j?i:j;
        for(k=0,p=pa+i*n-j,q=q0,t=0.; k<lc ;++k) t+= *p++ * *q++;
        q0[i]-=t;
       }
      for(i=0,q=q0,p=pa; i<n ;++i,p+=n) *p= *q++;
     }
    s=fabs(*pd); lc=j;
    for(k=j+1,ps=pd; k<n ;++k){
      if((t=fabs(*(ps+=n)))>s){ s=t; lc=k;}
     }
    tq=tq>s?tq:s; if(s<zr*tq){ free(le-j); free(q0); return -1;}
    *le++ =lc;
    if(lc!=j){
      for(k=0,p=a+n*j,q=a+n*lc; k<n ;++k){
        t= *p; *p++ = *q; *q++ =t;
       }
     }
    for(k=j+1,ps=pd,t=1./ *pd; k<n ;++k) *(ps+=n)*=t;
    *pd=t;
   }
  for(j=1,pd=ps=a; j<n ;++j){
    for(k=0,pd+=n+1,q= ++ps; k<j ;++k,q+=n) *q*= *pd;
   }
  for(j=1,pa=a; j<n ;++j){ ++pa;
    for(i=0,q=q0,p=pa; i<j ;++i,p+=n) *q++ = *p;
    for(k=0; k<j ;++k){ t=0.;
      for(i=k,p=pa+k*n+k-j,q=q0+k; i<j ;++i) t-= *p++ * *q++;
      q0[k]=t;
     }
    for(i=0,q=q0,p=pa; i<j ;++i,p+=n) *p= *q++;
   }
  for(j=n-2,pd=pa=a+n*n-1; j>=0 ;--j){ --pa; pd-=n+1;
    for(i=0,m=n-j-1,q=q0,p=pd+n; i<m ;++i,p+=n) *q++ = *p;
    for(k=n-1,ps=pa; k>j ;--k,ps-=n){ t= -(*ps);
      for(i=j+1,p=ps,q=q0; i<k ;++i) t-= *++p * *q++;
      q0[--m]=t;
     }
    for(i=0,m=n-j-1,q=q0,p=pd+n; i<m ;++i,p+=n) *p= *q++;
   }
  for(k=0,pa=a; k<n-1 ;++k,++pa){
    for(i=0,q=q0,p=pa; i<n ;++i,p+=n) *q++ = *p;
    for(j=0,ps=a; j<n ;++j,ps+=n){
      if(j>k){ t=0.; p=ps+j; i=j;}
      else{ t=q0[j]; p=ps+k+1; i=k+1;}
      for(; i<n ;) t+= *p++ *q0[i++];
      q0[j]=t;
     }
    for(i=0,q=q0,p=pa; i<n ;++i,p+=n) *p= *q++;
   }
  for(j=n-2,le--; j>=0 ;--j){
    for(k=0,p=a+j,q=a+ *(--le); k<n ;++k,p+=n,q+=n){
      t=*p; *p=*q; *q=t;
     }
   }
  free(le); free(q0);
  return 0;
}

void symm(double *a, int n)
{ int i,j;
 for(i=0; i<n ;i++)
   for(j=i+1; j<n ;j++)
     *(a+i*n+j) = (*(a+i*n+j) + *(a+j*n+i))/2;
 for(i=0; i<n ;i++)
   for(j=i+1; j<n ;j++)
     *(a+j*n+i) = *(a+i*n+j);
}

void minvsqrt(double *c, double *a, int n)
{
  double *eval = new double[n*n];
  double *evec = new double[n*n];
  for (int i=0; i<(n*n) ;i++) eval[i] = a[i];
  eejcb(eval, n, evec, 1.e-15, 6+5*n); 
  for (int i=0; i<n ;i++) eval[i] = 1./ sqrt(eval[(i*n) +i]);
 { double *p,*q,*ps,*r,*s,*t,*v=evec+n*n;
  for(ps=c,p=evec; p<v ;p+=n){
    for(q=evec; q<v ;q+=n,++ps){ *ps=0.;
      for(r=eval,s=p,t=q; r<eval+n ;)
        *ps+= *r++ * *s++ * *t++;
     }
   }
 }
 delete [] eval;
 delete [] evec;
}



} // namespace graphlib

/* end of file */
