/****************************************************************************
**
** Copyright (C) 2001 Hubert de Fraysseix, Patrice Ossona de Mendez.
** All rights reserved.
** This file is part of the PIGALE Toolkit.
**
** This file may be distributed under the terms of the GNU Public License
** appearing in the file LICENSE.HTML included in the packaging of this file.
**
*****************************************************************************/

#ifndef SPRINGEMBEDDER_H
#define SPRINGEMBEDDER_H

#include "utilities.h"
#include "graph.h"
#include "coordinates.h"
#include "eigenvector_eigenvalue.h"
#include "eigen.h"
#include "eigen2.h"
#include <algorithm>
#include "cyclegraph.h"
#include <vector>

namespace graphlib {



  double D_capEmbedder(Graph& origGraph, 
		       map<int,string>& hyb, map<Node,double>& charge,
		       bool embed)
  {
    const CycleGraph g(origGraph);

    NodeNumberMap ids;

    NodePathMap2 nodeMap;
    NodeSet nodeSet;
    constNodeIter r,i; unsigned r_,i_;
    unsigned sizeM = origGraph.numberNodes();  
    double* D = new double[sizeM*sizeM]; 
    double* D_cap = new double[sizeM*sizeM]; 
    
    r_ = 0;
    for (r = g.nodesBegin(); r != g.nodesEnd(); r++,r_++)
    {
      g.initNodeMap(*r,g,nodeMap,nodeSet);
      g.dijkstra(*r,nodeMap,nodeSet,true);
      i_ = 0;
      ids[*r] = r_;
      for (i = g.nodesBegin(); i != g.nodesEnd(); i++,i_++)
      {
        if (i == r)
	{
	  //D[(r->id()*sizeM)+i->id()] = 0;
	  D[(r_*sizeM)+i_] = 0;
          continue;
	}
        //D[(r->id()*sizeM)+i->id()] = nodeMap[*i].distance;
	//cout <<  D[(r->id()*sizeM)+i->id()] << "\t";
        D[(r_*sizeM)+i_] = 
	  nodeMap[*i].distance == 1 ? 1 : nodeMap[*i].distance * .9;
	//cout <<  D[(r_*sizeM)+i_] << "\t";
      }
      //cout << endl;
    }

    for (r = g.nodesBegin(); r != g.nodesEnd(); r++)
    {  
      for (constEdgeIter itE = r->adjBegin(); itE != r->adjEnd(); itE++)
      {
	constEdgeIter itE2 = itE; itE2++;
	for (; itE2 != r->adjEnd(); itE2++)
        {
	  if (hyb[r->id()] == "sp3")
	  {
	    D[(ids[r->opposite(*itE)]*sizeM)
	      +ids[r->opposite(*itE2)]] = 1.6332831;
	    D[(ids[r->opposite(*itE2)]*sizeM)
	      +ids[r->opposite(*itE)]] = 1.6332831;
	  } else if (hyb[r->id()] == "sp2") {
	    D[(ids[r->opposite(*itE)]*sizeM)
	      +ids[r->opposite(*itE2)]] =  1.7320508;
	    D[(ids[r->opposite(*itE2)]*sizeM)
	      +ids[r->opposite(*itE)]] =  1.7320508;
	  } else if (hyb[r->id()] == "sp1") {
	    D[(ids[r->opposite(*itE)]*sizeM)
	      +ids[r->opposite(*itE2)]] =  2.;
	    D[(ids[r->opposite(*itE2)]*sizeM)
	      +ids[r->opposite(*itE)]] =  2.;
	  }
	}
      }
    }

    //for (unsigned i = 0 ; i < sizeM*sizeM;i++) cout << D[i] << " ";

    double* Diag = new double[sizeM*sizeM];
    double* ev = new double[sizeM];
    eigen(D, ev, sizeM);
    for (unsigned i = 0; i < sizeM*sizeM; i++)
      Diag[i] = 0;
    for (unsigned i = 0; i < sizeM; i++)
      if (ev[i] > 0)
	Diag[(i*sizeM)+i] = ev[i];
    double* Dt = new double[sizeM*sizeM];
    double* tmp = new double[sizeM*sizeM];
    for (unsigned i = 0; i < sizeM*sizeM; i++)
      Dt[i] = D[i];
    mmul(tmp, D, Diag, sizeM);
    trnm(Dt,sizeM);
    mmul(D_cap, tmp, Dt, sizeM);
    for (unsigned i = 0; i < sizeM; i++)
      D_cap[(i*sizeM)+i] = 0;
    
    //for (unsigned i = 0 ; i < sizeM*sizeM;i++) cout << D_cap[i] << " ";
    delete[] Dt;
    delete[] tmp;
    delete[] D;

    //get dipole
    double dip = 0.;
    constNodeIter it = g.nodesBegin(); 
    constNodeIter it2;
    constNodeIter end = g.nodesEnd();
    for ( ; it != end; it++)
    {
      it2 = it; it2++;
      for (; it2 != end; it2++)
	dip += charge[*it]*charge[*it2]*D_cap[(ids[*it]*sizeM)+ids[*it2]];
    }

    delete[] ev;
    delete[] Diag;

    if (!embed)
      return ( -sqrt(abs(dip)) * dip / abs(dip) );

    // transform into principal coordinates
    double d2_sum = 0;
    for (unsigned i = 0; i < sizeM ; i++)
      for (unsigned j = 0; j < i; j++)
	d2_sum += D_cap[(i*sizeM)+j]*D_cap[(i*sizeM)+j];

    d2_sum /= sizeM*sizeM;

    double* d2_c = new double[sizeM];
    double tmp2;
    for (unsigned i = 0; i < sizeM ; i++)
    {
      tmp2 = 0.;
      for (unsigned j = 0; j < i; j++)
	tmp2 += D_cap[(i*sizeM)+j]*D_cap[(i*sizeM)+j];
      for (unsigned j = i+1; j < sizeM; j++)
	tmp2 += D_cap[(j*sizeM)+i]*D_cap[(j*sizeM)+i];
      d2_c[i] = (tmp2/sizeM) - d2_sum; //negative?
    }

    double* metric_M = new double[sizeM*sizeM];
    for (unsigned i = 0; i < sizeM ; i++)
      for (unsigned j = 0; j < sizeM; j++)
	metric_M[(i*sizeM)+j] = (d2_c[i] + d2_c[j] - D_cap[(i*sizeM)+j])/2.;

    //eigen of metric
    ev = new double[sizeM];
    eigen(metric_M, ev, sizeM);
    unsigned k,l,m;
    if (ev[0] > ev[1])
    { if (ev[1] > ev[2])
      { k = 0; l = 1; m = 2; 
        } else if (ev[0] > ev[2])
        { k = 0; m = 1; l = 2; 
        } else { l = 0; m = 1; k = 2; }
    } else {
      if (ev[2] > ev[1])
      { m = 0; l = 1; k = 2; 
        } else if (ev[2] > ev[0])
	  { m = 0; k = 1; l = 2; 
	  } else { l = 0; k = 1; m = 2; }
    } 
    for (unsigned i = 3; i < sizeM; i++)
    { 
      if (ev[i] > ev[k]) {
	m = l; l = k; k = i; 
      } else if (ev[i] > ev[l]) {
	m = l; l = i; 
      } else if (ev[i] > ev[m]) {
	m = i;
      }
    }
    //coord = sqrt(eval)*evec
    i_ = 0; 
    for (constNodeIter v = origGraph.nodesBegin(); v != origGraph.nodesEnd(); v++,i_++)
      {Coordinates p;
      p.x = sqrt(ev[k])*metric_M[(i_*sizeM)+k];
      p.y = sqrt(ev[l])*metric_M[(i_*sizeM)+l];
      p.z = sqrt(ev[m])*metric_M[(i_*sizeM)+m];
      origGraph.findNode(v->id())->coord(p);
      }

    delete[] d2_c;
    delete[] metric_M;
    delete[] ev;
    delete[] D_cap;

    return ( -sqrt(abs(dip)) * dip / abs(dip) );
  }


int ComputeBounds(Graph &g,double & dx,double & dy, double & dz)
{//G.MinMaxCoords(xmin,xmax,ymin,ymax);
  double xmax = g.nodesBegin()->coord().x;
  double ymax = g.nodesBegin()->coord().y;
  double zmax = g.nodesBegin()->coord().z;
  double xmin = g.nodesBegin()->coord().x;
  double ymin = g.nodesBegin()->coord().y;
  double zmin = g.nodesBegin()->coord().z;
 for (constNodeIter v = g.nodesBegin(); v != g.nodesEnd(); v++)
    {const Coordinates& p = v->coord();
    xmax = xmax > p.x ? xmax : p.x;
    ymax = ymax > p.y ? ymax : p.y;
    zmax = zmax > p.z ? zmax : p.z;

    xmin = xmin < p.x ? xmin : p.x;
    ymin = ymin < p.y ? ymin : p.y;
    zmin = zmin < p.z ? zmin : p.z;
    }
  dx = xmax - xmin;
  dy = ymax - ymin;
  dz = zmax - zmin;
  return 0;
  } 
char sgn(double f)
{ if (f==0) return('0');
 if (f<0) return('-'); else return('+');}

double dist_seg(const Coordinates &p0,const Coordinates &p1,const Coordinates &p2,Coordinates &pmin)
// retourne distance au carre du point p0 au segment (p1,p2)
// p realize the shortest distance
    {double scal =  ((p1.x-p0.x)*(p1.x-p2.x))+
                    ((p1.y-p0.y)*(p1.y-p2.y))+
                    ((p1.z-p0.z)*(p1.z-p2.z));  
    double d_12 =  ((p1.x-p2.x)*(p1.x-p2.x))+
                   ((p1.y-p2.y)*(p1.y-p2.y))+
                   ((p1.z-p2.z)*(p1.z-p2.z));
    double d_10 =  ((p1.x-p0.x)*(p1.x-p0.x))+
                   ((p1.y-p0.y)*(p1.y-p0.y))+
                   ((p1.z-p0.z)*(p1.z-p0.z));
    pmin = p1;
    if(d_12 == 0. || scal < 0)//distance a p1
        {return d_10;}  
    else if(scal > d_12)//distance a p2
        {pmin = p2;
        return(d_10 - 2.*scal + d_12);
        }     
    else //entre p1 et p2
        {pmin =  p1;
	pmin.x += (scal/d_12)*(p2.x - p1.x);
	pmin.y += (scal/d_12)*(p2.y - p1.y);
	pmin.z += (scal/d_12)*(p2.z - p1.z);
        return(d_10 - scal*scal/d_12);
        }     
    }

#define BORDER 20

void springEmbedder(Graph& g, int h, int w, int d, bool init = true)
{
  vector<Coordinates> translate(1,g.numberNodes()); translate.clear();
  double mhw = minimum(w,h) - 2*BORDER;
  Coordinates center;
  center.x =(w - 1 -12)/2.;//space - sizerect)/2.;
  center.y =h/2.;
  center.z =d/2.;
  int n_red,n = g.numberNodes(),m =g.numberEdges();
  double len,len02 = mhw*mhw/n;
  // during iteration keeep the drawing size
  double expand = 1.;
  double hw,hw0 = .25*(mhw*mhw)/(n*m);//.5
  int iter,niter = 2000;
  //int iter,niter = 1;
  double dist2,strength,dx,dy,dz,dep;
  //double xmin,xmax,ymin,ymax,zmin,zmax,
  double sizex,sizey,sizez,sizex0,sizey0,sizez0;
  Coordinates p,translation;//p0
  double force = 1.;
  int stop = 0;
  
  if (init)
    {unsigned i = 0;
    for (constNodeIter v = g.nodesBegin(); v != g.nodesEnd(); v++,i++)
      {Coordinates p;
      p.x = center.x + (mhw*cos(2*M_PI/g.numberNodes()*i)/2.);
      p.y = center.y + (mhw*sin(2*M_PI/g.numberNodes()*i)/2.);
      p.z = i&1 ? center.z + d/4 : center.z - d/4;
      g.findNode(v->id())->coord(p);
      }
    }
  
  // Compute bounds
  ComputeBounds(g,sizex0,sizey0, sizez0); 
  
  for(iter = 1;iter <= niter;iter++)
      {translate.clear();
      if(iter > 50)force *= .99;
      else if(iter > 100)force *= .98;
      // Compute mean length of edges
      len = 1;
      len02 = len*len;
      hw = expand*hw0;
      for(constNodeIter v0 = g.nodesBegin(); v0 != g.nodesEnd();v0++)
	{Coordinates p0 = v0->coord(); //WATCHOUT!!!
          translation.x = translation.y = translation.z = 0.;
          // v0 repulse other vertices (1/d)
          for(constNodeIter v = g.nodesBegin(); v != g.nodesEnd();v++)
              {if(v == v0)continue;
              const Coordinates& p = v->coord();
              dist2 = ((p.x-p0.x)*(p.x-p0.x)) + 
		      ((p.y-p0.y)*(p.y-p0.y)) + 
		      ((p.z-p0.z)*(p.z-p0.z)) ;
	      dist2 = dist2 > 1. ? dist2 : 1.;
              strength = (hw/dist2);
              translation.x  += (p0.x - p.x)*strength;
	      translation.y  += (p0.y - p.y)*strength;
              translation.z  += (p0.z - p.z)*strength;
             }
          // v0 is repulsed by non adjacent edges (1/d)
          // if too small cir changes
          for(constEdgeIter e = g.edgesBegin(); e != g.edgesEnd();e++)
              {const Node& v = e->source(), w = e->target();
              if(*v0 == v || *v0 == w)continue;
              dist2 = dist_seg(p0,v.coord(),w.coord(),p);
              if(dist2 > 2.)
                  {strength = (hw/dist2); 
		  translation.x  += (p0.x - p.x)*strength;
		  translation.y  += (p0.y - p.y)*strength;
		  translation.z  += (p0.z - p.z)*strength;   
                  }
              else if(v.coord().y != v.coord().y)
                  translation.x += 5.;
              else
                  translation.y += 5.;
              }
          //v0 is attracted or repulsed by its neighbours (1/d)
	  /*
          tbrin b0 = G.pbrin[v0];
          tbrin b = b0;
          do
              {p = G.vcoord[G.vin[-b]];
              dist2 = Max(Distance2(p0,p),1.);
	  */
	  for (constEdgeIter b = v0->adjBegin(); b!= v0->adjEnd(); b++)
	      {const Coordinates& p = v0->opposite(*b).coord();
              dist2 = ((p0.x-p.x)*(p0.x-p.x)) +
		      ((p0.y-p.y)*(p0.y-p.y)) +
		      ((p0.z-p.z)*(p0.z-p.z));
	      dist2 = dist2 > 1. ? dist2 : 1. ;
	      strength = sqrt(hw/dist2);
	      strength = strength > 1. ? 1 : strength;
              if(dist2 > len02/4)
                  {translation.x  -= (p0.x-p.x)*strength*.5;
		  translation.y  -= (p0.y-p.y)*strength*.5;
		  translation.z  -= (p0.z-p.z)*strength*.5;
              } else if(dist2 < 4*len02) 
                  {translation.x  += (p0.x-p.x)*strength*.5;
		  translation.y  += (p0.y-p.y)*strength*.5;
		  translation.z  += (p0.z-p.z)*strength*.5;
		  }
	    } //while((b = G.cir[b]) != b0);
          // v0 is attracted by the center (1/d)
          dist2 =  ((p0.x-center.x)*(p0.x-center.x)) +
		   ((p0.y-center.y)*(p0.y-center.y)) +
		   ((p0.z-center.z)*(p0.z-center.z));
	  dist2 = dist2 > 1. ? dist2 : 1. ;
          strength = sqrt(hw/dist2);
	  strength = strength > .5 ? .5 : strength;
	  strength *= .5;
          translate[v0->id()].x -= (p0.x - center.x)*strength;
          translate[v0->id()].y -= (p0.y - center.y)*strength;
          translate[v0->id()].z -= (p0.z - center.z)*strength;
          translation.x -= (p0.x - center.x)*strength;
          translation.y -= (p0.y - center.y)*strength;
          translation.z -= (p0.z - center.z)*strength;
          // update v0
          translation.x *= force;
          translation.y *= force;
          translation.z *= force;
          translate[v0->id()] = translation;
          p0.x += translation.x;
          p0.y += translation.y;
          p0.z += translation.z;
	  g.findNode(v0->id())->coord(p0);
          }

      // update the drawing
      dep = .0;
      n_red = 0;
      for(constNodeIter v = g.nodesBegin(); v != g.nodesEnd();v++)
          {dx = abs(translate[v->id()].x); dy = abs(translate[v->id()].y);
	  dz =  abs(translate[v->id()].z);
          dep = dep > dx ? dep : dx;  dep =  dep > dy ? dep : dy;
	  dep = dep > dz ? dep : dz;
	  /*
          if(dx > 1. || dy > 1. || dz > 1.) 
              {nodeitem[v]->SetColor(color[G.vcolor[v]]);
              //nodeitem[v]->moveTo(G.vcoord[v]);
              }
          else
              {nodeitem[v]->SetColor(red);++n_red;}
          nodeitem[v]->moveTo(G.vcoord[v]);
          canvas()->update();
          }*/
      //stop = (n_red >= (2*G.nv())/3)? ++stop : 0;
      stop = (n_red == (int)g.numberNodes())? ++stop : 0;
      if(stop)force *= .9;
      if(dep < .25 || stop == 4)break;
      /*qApp->processEvents(1);
      //if(gwp->mywindow->getKey() == Qt::Key_Escape)break;
      if(key_pressed == Qt::Key_Escape)break;*/
      // Compute bounds ro adapt the expand factor (should not be too strong)
      ComputeBounds(g,sizex,sizey,sizez); 
      if(sizex > sizex0 && sizey > sizey0 && sizez > sizez0)
          expand /= -minimum(-sizex/sizex0,minimum(-sizey/sizey0,-sizez/sizez0));
      else if(sizex < sizex0 && sizey < sizey0 && sizez < sizez0)
          expand *= -minimum(-sizex/sizex0,minimum(-sizey/sizey0,-sizez/sizez0));
      sizex0 = sizex;      sizey0 = sizey; sizez0 = sizez;
      }
  /*
  Normalise();
  // same as load(false) but much faster
  for(tvertex v = 1;v <= n;v++)
      {nodeitem[v]->SetColor(color[G.vcolor[v]]);
      nodeitem[v]->moveTo(G.vcoord[v]);
      }
  canvas()->update();
#ifdef  VERSION_ALPHA 
  if(debug())
      Tprintf("Iter=%d len=%d stop=%d dep=%f expand=%f force=%f",iter,(int)len,stop,dep,expand,force);
#endif
  */
      }

}
}

#endif
